@@ -4,6 +4,15 @@ if (!isGeneric("bbox"))
 if (!isGeneric("coordinates"))
 	setGeneric("coordinates", function(obj)
 		standardGeneric("coordinates"))
+if (!isGeneric("coordinates<-"))
+	setGeneric("coordinates<-", function(object, value)
+		standardGeneric("coordinates<-"))
+if (!isGeneric("coordnames"))
+	setGeneric("coordnames", function(x)
+		standardGeneric("coordnames"))
+if (!isGeneric("coordnames<-"))
+	setGeneric("coordnames<-", function(x,value)
+		standardGeneric("coordnames<-"))
 if (!isGeneric("rings"))
 	setGeneric("rings", function(obj)
 		standardGeneric("rings"))
